class Compute:
    # CPU Tiers
    CPU_SMALL = "cpu-small"
    CPU_MEDIUM = "cpu-medium"
    CPU_LARGE = "cpu-large"
    CPU_XL = "cpu-xl"

    # GPU Tiers
    # H200s
    H200_SMALL = "h200-small"
    H200_MEDIUM = "h200-medium"
    H200_LARGE = "h200-large"
    H200_XL = "h200-xl"
    # H100s
    H100_SMALL = "h100-small"
    H100_MEDIUM = "h100-medium"
    H100_LARGE = "h100-large"
    H100_XL = "h100-xl"
    # RTX4090s
    RTX4090_SMALL="rtx4090-small"
    RTX4090_MEDIUM="rtx4090-medium"
    RTX4090_LARGE="rtx4090-large"