# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import function_execution_pb2 as function__execution__pb2

GRPC_GENERATED_VERSION = '1.76.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower

    _version_not_supported = first_version_is_lower(
        GRPC_VERSION, GRPC_GENERATED_VERSION
    )
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + ' but the generated code in function_execution_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class FunctionExecutorStub(object):
    """Function execution service for Targon containers"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Execute = channel.unary_unary(
            '/targon.runtime.FunctionExecutor/Execute',
            request_serializer=function__execution__pb2.ExecuteRequest.SerializeToString,
            response_deserializer=function__execution__pb2.ExecuteResponse.FromString,
            _registered_method=True,
        )


class FunctionExecutorServicer(object):
    """Function execution service for Targon containers"""

    def Execute(self, request, context):
        """Execute a function with serialized arguments"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FunctionExecutorServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'Execute': grpc.unary_unary_rpc_method_handler(
            servicer.Execute,
            request_deserializer=function__execution__pb2.ExecuteRequest.FromString,
            response_serializer=function__execution__pb2.ExecuteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'targon.runtime.FunctionExecutor', rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers(
        'targon.runtime.FunctionExecutor', rpc_method_handlers
    )


# This class is part of an EXPERIMENTAL API.
class FunctionExecutor(object):
    """Function execution service for Targon containers"""

    @staticmethod
    def Execute(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/targon.runtime.FunctionExecutor/Execute',
            function__execution__pb2.ExecuteRequest.SerializeToString,
            function__execution__pb2.ExecuteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )


class HealthStub(object):
    """Health check service for Knative probes"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Check = channel.unary_unary(
            '/targon.runtime.Health/Check',
            request_serializer=function__execution__pb2.HealthCheckRequest.SerializeToString,
            response_deserializer=function__execution__pb2.HealthCheckResponse.FromString,
            _registered_method=True,
        )
        self.Watch = channel.unary_stream(
            '/targon.runtime.Health/Watch',
            request_serializer=function__execution__pb2.HealthCheckRequest.SerializeToString,
            response_deserializer=function__execution__pb2.HealthCheckResponse.FromString,
            _registered_method=True,
        )


class HealthServicer(object):
    """Health check service for Knative probes"""

    def Check(self, request, context):
        """Check if the service is healthy"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Watch(self, request, context):
        """Watch for health status changes"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HealthServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'Check': grpc.unary_unary_rpc_method_handler(
            servicer.Check,
            request_deserializer=function__execution__pb2.HealthCheckRequest.FromString,
            response_serializer=function__execution__pb2.HealthCheckResponse.SerializeToString,
        ),
        'Watch': grpc.unary_stream_rpc_method_handler(
            servicer.Watch,
            request_deserializer=function__execution__pb2.HealthCheckRequest.FromString,
            response_serializer=function__execution__pb2.HealthCheckResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'targon.runtime.Health', rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('targon.runtime.Health', rpc_method_handlers)


# This class is part of an EXPERIMENTAL API.
class Health(object):
    """Health check service for Knative probes"""

    @staticmethod
    def Check(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/targon.runtime.Health/Check',
            function__execution__pb2.HealthCheckRequest.SerializeToString,
            function__execution__pb2.HealthCheckResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )

    @staticmethod
    def Watch(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/targon.runtime.Health/Watch',
            function__execution__pb2.HealthCheckRequest.SerializeToString,
            function__execution__pb2.HealthCheckResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True,
        )
